#pragma once
#include <kr2_program_api/internals/api/console.h>
#include "communication/bridge/connection_lifecycle/i_monitor.h"
#include "util/i_scheduled_executor.h"
#include "communication/bridge/i_bridge_client.h"
#include "communication/bridge/connection_lifecycle/i_heartbeat_handler.h"
#include "communication/bridge/connection_lifecycle/i_state_handler.h"

namespace spin
{
    using namespace std::chrono_literals;

    class Monitor : public IMonitor
    {
    public:
        Monitor(
            std::shared_ptr<IScheduledExecutor> executor,
            std::shared_ptr<IBridgeClient> bridgeClient,
            std::shared_ptr<IHeartbeatHandler> heartbeatHandler,
            std::shared_ptr<IStateHandler> stateHandler);
        ~Monitor();

        void monitorHeartbeat() override;
        void monitorEverything() override;
        void stopMonitoring() override;
        void setClientExceptionCallback(std::function<void()> callback);

    private:
        void onBridgeConnected();
        void monitorItems();
        void fetchAndProcessMonitoredItems();

        static constexpr auto initialSchedulingDelay = 1000ms;
        static constexpr auto schedulingPeriod = 50ms;

        bool monitorHeartbeatIsScheduled_ = false;
        bool monitorEverythingIsScheduled_ = false;
        std::shared_ptr<IScheduledExecutor> executor_;
        std::shared_ptr<IBridgeClient> bridgeClient_;
        std::shared_ptr<IHeartbeatHandler> heartbeatHandler_;
        std::shared_ptr<IStateHandler> stateHandler_;
        std::function<void()> clientExceptionCallback_;
    };

} // namespace spin